'    Jump, one-button game
'    Copyright (C) 2010  Pavle Simovic
'
'    This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'
'    Author can be contacted at xpio<at>tut.by

'    FreeBASIC code below

#include "fbgfx.bi"

const scr_width=200, scr_height=200
const sprite_width=80, sprite_height=26
const cell_width=26

const amortization=-5
const maxcycles=200
const drawcell_x=(scr_width-cell_width)/2
const drawcell_y=(scr_height-sprite_height)/2
const as double pi = 3.1415926535897932

type rope
	as integer phase
	as double angle
	'angle=-1 disables rope
	'angle=-2 introduces rope in next turn
end type

dim shared as rope ropes(20)
dim shared as any ptr sprite_img
dim shared as integer jumper, cycle=0, level=0
dim shared as byte message

sub drawcell(n as ubyte)
	put (drawcell_x, drawcell_y), sprite_img, _
	(n*cell_width, 0)-((n+1)*cell_width-1, sprite_height), trans
end sub

sub init
	dim a as ubyte
	screenres scr_width,scr_height,8,2
	screenset 2,1 
	sprite_img=imagecreate(sprite_width,sprite_height,0,8)
	bload exepath+"/jump.bmp", sprite_img
	jumper=0
	for a=0 to 19
		ropes(a).angle=-1
	next a
	randomize timer
	message=2
end sub

sub stretchline(x1 as integer ,y1 as integer , _
x2 as integer ,y2 as integer , c as ubyte=20)
	const margin=71 'to prevent cases where both points are outside
	if x1=x2 and y1=y2 then return
	dim as integer startx=x1, starty=y1, endx=x2, endy=y2
	do while startx>0-margin and starty>0-margin and _
	startx<scr_width+margin and starty<scr_height+margin
		startx-=x2-x1
		starty-=y2-y1
	loop
	do while endx<scr_width+margin and endy<scr_height+margin and _
	endx>0-margin and endy>0-margin
		endx+=x2-x1
		endy+=y2-y1
	loop
	line (startx,starty)-(endx,endy),c
end sub

const distancehorizon=142
const ropeacceleration=1.1
sub drawrope(distance as integer, angle as double, col as ubyte=20)
	dim as double x1, y1, x2, y2, h
	'transform distance to get acceleration
	distance=fix((1/(1+ropeacceleration^(-1*distance))-0.5)*distancehorizon*2)
	x1=cos(angle)*distance+scr_width/2
	y1=sin(angle)*distance+scr_height/2
	x2=cos(angle+pi/2)*100+x1
	y2=sin(angle+pi/2)*100+y1
	stretchline(x1,y1,x2,y2, col)
end sub

sub drawshadow
	static as ushort jumplength
	dim shadowdistance as ushort
	if jumper<0 then
		jumplength=0
		circle (scr_width/2+3, scr_height/2+2), 11, 29, , , ,F
		circle (scr_width/2+10, scr_height/2+5), 9, 29, , ,0.8 ,F
	elseif jumper=0 then
		jumplength=0
		circle (scr_width/2+5, scr_height/2+3), 11, 29, , ,0.8 ,F
		circle (scr_width/2+13, scr_height/2+6), 9, 29, , ,0.8 ,F
	else
		if jumplength=0 then jumplength=jumper
		shadowdistance=sin((jumplength-jumper)/jumplength*pi)*jumplength
		circle (scr_width/2+5+shadowdistance*2.3, _
		scr_height/2+3+shadowdistance*0.8), 11, 29, , ,0.8 ,F
		circle (scr_width/2+shadowdistance*2.2+14, _
		scr_height/2+shadowdistance*0.75+7), 9, 29, , ,0.8 ,F
	end if
end sub

sub drawworld
	cls
	if message=1 then
		print "wow!"
	elseif message=2 then
		print "press space to jump"
	else
		print "level";level
	end if
	'shadow
	drawshadow
	'ropes
	dim as ubyte a
	for a=0 to 19
		if ropes(a).angle>-1 then
			drawrope(cycle-ropes(a).phase,ropes(a).angle)
			drawrope(cycle-ropes(a).phase-maxcycles,ropes(a).angle)
			drawrope(cycle-ropes(a).phase+maxcycles,ropes(a).angle)
		elseif ropes(a).angle=-2 and (cycle=ropes(a).phase+maxcycles/2 or _
		cycle=ropes(a).phase-maxcycles/2) then
			ropes(a).angle=rnd*2*pi
		end if
	next a
	'jumper
	if jumper>0 then
		drawcell(0)
	elseif jumper=0 then
		drawcell(1)
	else
		drawcell(2)
	end if
	'
	flip
end sub

'jumper=0 - girl is standing
'jumper<0 - girl is crouching
'-20<jumper<0 - girl is crouching after jump, returning to standing position
'jumper > 0 - girl is in the air

sub managejumper
	if multikey(FB.SC_SPACE) then
		if jumper<=0 and jumper>amortization then
			jumper=amortization
		elseif jumper<=amortization then
			jumper-=1
		end if
	elseif jumper<=amortization then 'if spent some time crouching
		jumper=jumper*-1 'go to air for as many cycles
	end if
	if jumper>1 then 'fall
		jumper-=1
	elseif jumper=1 then 'crouch after jump, for 20 cycles
		jumper=amortization+1
	elseif jumper>amortization and jumper<0 then
		jumper+=1
	end if
end sub

sub scenario
'count cycles with static variable
'introduce new ropes
	static as integer times
	dim as byte a, found
	if times=0 or times > level+1 then
		times=0
		for a=0 to 19
			found=ropes(a).angle=-1
			if found then
				level+=1
				ropes(a).angle=-2 'disabled until safe
				ropes(a).phase=fix(rnd*maxcycles)
				exit for
			end if
		next a
		if not found then message=1 'say wow
	end if
	times+=1
end sub

sub detectcollision
	if jumper>0 then return
	dim as integer a
	for a=0 to 19
		if ropes(a).angle>-1 and ropes(a).phase=cycle then
			jumper=-5
			if level=1 then
				ropes(a).angle=-2
			else
				ropes(a).angle=-1
				level-=1
			end if
		end if
	next a
end sub

sub main
	dim as string keypressed
	scenario
	do
		cycle+=1
		if cycle>maxcycles then
			cycle=0
			scenario
		end if
		managejumper
		detectcollision
		drawworld
		keypressed=inkey
		'clear "Press space to jump"
		if message=2 and keypressed=chr(32) then message=0
		sleep 30
	loop until keypressed=chr(27) or keypressed=chr(255)+"k"
end sub

sub quit
	imagedestroy sprite_img
end sub

init
main
quit
